
/* 

Program: modfid - allows one to modify a fid in a n-dimensional experiment 
Author - Robert Boyko, Jan 2007
Usage: modfid <data> <fid_number> <function>
where <function> is average or zero or whatever you make up.
Example modfid test.fid 233 zero

Notes: I found the od command helpful for debugging how the binary fids were
constructed.
#od -t dL -N 100 olga.fid

*/

#include <stdio.h>
#define SEEK_CUR 1
#define FID_READ_NUM 5

/* define fid structures */
struct file_header {
	long nblocks, ntraces, np, ebytes, tbytes, bbytes;
	short vers_id, status;
	long nBlockheaders;
};
struct block_header {
	short scale, status, index, mode;
	long ctcount;
	float lpval, rpval, lvl, tlt;
};

int fidKey, fidStart, fidEnd;
char *fidFile, *fidFunc;
FILE *fPtr;

struct file_header  fileHeader;
struct block_header  *blockHeader[FID_READ_NUM];
int *fidData[FID_READ_NUM];
int *newData[FID_READ_NUM];

extern int calloc();

/********************************************************************
* Main program
*/
int main (argc, argv )
int argc;
char *argv[];
{
	int nd;

	check_arguments(argc, argv);
	read_data_file(fPtr);

	if (strcmp(fidFunc, "average") == 0) {
		ave_fid();
	} else {
		if (strcmp(fidFunc, "zero") == 0) {
			zero_fid();
		} else {
			fprintf( stderr, "Error, do not know function '%s'\n", fidFunc);
			return;
		}
	}

	write_data_file(fPtr);
	printf("Terminating ok...\n\n");
}

/********************************************************************
* Check for valid command line arguments 
*/
int check_arguments (argc, argv)
int argc;
char *argv[];
{
	int nd;

	/* check number of arguments */
	if ( argc != 4 ) {
		fprintf( stderr, "Usage: modfid  fid_file fid_number [average zero]\n");
		exit(1);
	}

	/* assign reasonable names to command line arguments */
	fidFile = argv[1];
	nd = sscanf(argv[2], "%d", &fidKey );
	fidFunc = argv[3];

	/* check for valid fid number */
	if (( !nd ) || fidKey < 1 ) {
		fprintf( stderr, "Error, fid number <= 0\n" );
		return;
	}

	/* make sure we can open the data file and write to it */
	fPtr = fopen(fidFile, "r+");
	if ( fPtr == NULL ) {
		fprintf( stderr, "Error: unable to open file '%s'\n", fidFile);
		return;
	}
	printf("Arguments ok...\n");
	printf("Fid File is '%s'\n", fidFile);
}

/********************************************************************
* Read the fid data 
*/
int read_data_file(fp)
FILE *fp;
{
	int i, j;

	/* read the file header */
	rewind(fp);
	fread(&fileHeader, sizeof(fileHeader), 1, fp);
	printf( "Number of blocks = %d\n", fileHeader.nblocks);
	printf( "Number of points = %d\n", fileHeader.np);
	printf( "Status  = %d\n", fileHeader.status);

	/* now we know how big each fid is, allocate space to read data */
	for ( i = 0; i < FID_READ_NUM; i++ ) {
		fidData[i] = ( int * )calloc(fileHeader.np, fileHeader.ebytes);
		blockHeader[i] = ( int * )calloc(7, fileHeader.ebytes);
		newData[i] = ( int * )calloc(fileHeader.np, fileHeader.ebytes);
	}

	/* decide which fids we need to read for possible processing */
	/* check for endpoints */
	fidStart = fidKey - (FID_READ_NUM / 2); 
	if (fidStart <= 0) fidStart = 1;
	fidEnd = fidKey + (FID_READ_NUM / 2); 
	if (fidEnd >= fileHeader.nblocks) fidEnd = fileHeader.nblocks;
	printf( "Reading fids %d to %d\n", fidStart, fidEnd);

	/* move file pointer to first fid to read */
	fseek(fp, (fidStart - 1) * fileHeader.bbytes, SEEK_CUR );

	/* read fid data, ignore fid header info for now */
	for ( i = 0; i <= (fidEnd - fidStart); i++ ) {
		fread(blockHeader[i], fileHeader.ebytes, 7, fp);
		fread(fidData[i], fileHeader.ebytes, fileHeader.np, fp);
		printf( "block %4d Starts at: %d %d ...\n", i+fidStart, blockHeader[i][0], blockHeader[i][1]);
		printf( "Fid %4d Starts at: %d %d ...\n", i+fidStart, fidData[i][0], fidData[i][1]);
		for (j = 0; j <= fileHeader.np; j++ ) {
			newData[i][j] = fidData[i][j];
		}
	}
	printf( "Fids Read ok...\n");
}

/********************************************************************
* Replace selected fid with an averaged fid 
*/
int ave_fid()
{
	int i, prev, next, ix;

	/* use fids on either side of the keyfid to fix */
	ix = fidKey - fidStart;
	prev = ix - 2;
	next = ix + 2;
	printf( "Computing average using fids %d and %d\n", fidKey-1, fidKey+1);

	for ( i = 0; i < fileHeader.np; i++ ) {
		newData[ix][i] = 0.5*(fidData[prev][i] + fidData[next][i]);
	}
	printf( "Average fid done...\n");
}

/********************************************************************
* Zero selected fid
*/
int zero_fid()
{
	int i, ix;

	ix = fidKey - fidStart;
	printf( "Zero fid %d \n", fidKey);

	for ( i = 0; i < fileHeader.np; i++ ) {
		newData[ix][i] = 0;
	}
	printf( "Zero fid done...\n");
}

/********************************************************************
* write the modified fid 
*/
int write_data_file(fp)
FILE *fp;
{
	int i;

	/* read the file header */
	rewind(fp);
	fread(&fileHeader, sizeof(fileHeader), 1, fp);

	/* move file pointer to fid */
	fseek(fp, (fidStart - 1) * fileHeader.bbytes, SEEK_CUR );

	/* write out modified fid array and close file */
	printf( "Writing fids %d to %d\n", fidStart, fidEnd);
	for ( i = 0; i <= (fidEnd - fidStart); i++ ) {
		fwrite(blockHeader[i], fileHeader.ebytes, 7, fp);
		fwrite(newData[i], fileHeader.ebytes, fileHeader.np, fp);
		printf( "header %4d Starts at: %d %d ...\n", i+fidStart, blockHeader[i][0], blockHeader[i][1]);
		printf( "Fid %4d Starts at: %d %d ...\n", i+fidStart, newData[i][0], newData[i][1]);
	}
	fclose(fp);
	printf("File Updated ok: %s\n", fidFile);
}
